package laboratoire4;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class MiniMax {
    private static final int MAX_DEPTH = 4;
    private static Random random = new Random();

    public static MiniMaxResult miniMax(PusherBoard board) {
        return miniMax(board, true, 0, Integer.MIN_VALUE);
    }

    private static MiniMaxResult miniMax(PusherBoard board, boolean max, int depth, int alphaBeta) {
        int limScore = max ? Integer.MIN_VALUE : Integer.MAX_VALUE;
        List<Pawn> pawns = max ?
                board.getMaxPawns() :
                board.getMinPawns();

        List<MiniMaxResult> results = new ArrayList<>();

        for (Pawn pawn : pawns) {
            int originalRow = pawn.getRow();
            int originalCol = pawn.getCol();

            for (Pawn.PawnMovement movement : Pawn.PawnMovement.values()) {
                if (pawn.isMoveValid(board, movement)) {
                    pawn.move(movement);

                    int score = depth < MAX_DEPTH ?
                            miniMax(board, !max, depth + 1, limScore).getScore() :
                            evaluate(pawn, board);
                    score *= pawn.getDirection();

                    if ((max && score > limScore) ||
                            (!max && score < limScore)) {
                        limScore = score;
                    }

                    MiniMaxResult result = new MiniMaxResult(limScore, pawn, movement);

                    //elagage alphaBeta
                    if ((max && limScore >= alphaBeta) ||
                            (!max && limScore <= alphaBeta)) {
                        pawn.setRow(originalRow);
                        pawn.setCol(originalCol);

                        return result;
                    }

                    results.add(result);
                }

                pawn.setRow(originalRow);
                pawn.setCol(originalCol);
            }
        }

        // Choisir aléatoirement
        int index = random.nextInt(results.size());
        return results.get(index);
    }

    private static int evaluate(Pawn pawn, PusherBoard board) {
        int score = didWin(pawn);
        score = Math.max(score, didCapture(pawn, board));

        return score;
    }
}
